@@ -18,10 +18,8 @@ limitations under the License.
 import logging
 
 from cumulus_ds import bundle_manager
-from cumulus_ds import config as cfg
 from cumulus_ds import deployment_manager
-
-config = cfg.Configuration()
+from cumulus_ds.config import CONFIG as config
 
 LOGGING_CONFIG = {
     'version': 1,
@@ -59,22 +57,27 @@ LOGGING_CONFIG = {
             'level': logging.CRITICAL,
             'propagate': False
         },
-        'lib.bundle_manager': {
+        'cumulus_ds.bundle_manager': {
             'handlers': ['default'],
             'level': 'DEBUG',
             'propagate': False
         },
-        'lib.config_handler': {
+        'cumulus_ds.config_handler': {
             'handlers': ['default'],
             'level': 'DEBUG',
             'propagate': False
         },
-        'lib.connection_handler': {
+        'cumulus_ds.connection_handler': {
             'handlers': ['default'],
             'level': 'DEBUG',
             'propagate': False
         },
-        'lib.deployment_manager': {
+        'cumulus_ds.deployment_manager': {
+            'handlers': ['default'],
+            'level': 'DEBUG',
+            'propagate': False
+        },
+        'cumulus_ds.helpers.stack': {
             'handlers': ['default'],
             'level': 'DEBUG',
             'propagate': False
@@ -114,6 +117,9 @@ def main():
         if config.args.events:
             deployment_manager.list_events()
 
+        if config.args.outputs:
+            deployment_manager.list_outputs()
+
     except Exception as error:
         LOGGER.error(error)
         raise
